local LayoutHelpers = import('/lua/maui/layouthelpers.lua') 
local Button = import('/lua/maui/button.lua').Button
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local ItemList = import('/lua/maui/itemlist.lua').ItemList
local Group = import('/lua/maui/group.lua').Group
local UIUtil = import('/lua/ui/uiutil.lua')
local GameCommon = import('/lua/ui/game/gamecommon.lua')
local ToolTip = import('/lua/ui/game/tooltip.lua')
local TooltipInfo = import('/lua/ui/help/tooltips.lua').Tooltips
local AvatarsClickFunc = import('/lua/ui/game/avatars.lua').ClickFunc
local GameMain = import('/lua/ui/game/gamemain.lua')

local modFolder = 'MexManager'
local CreateGrid = import('/mods/' .. modFolder .. '/modules/mcibuttons.lua').CreateGrid
local CreateGenericButton = import('/mods/' .. modFolder .. '/modules/mcibuttons.lua').CreateGenericButton
local mexes = {}
local units = {}
--local updateThread
local DelayUpdateMexes = 0
local AutoUpdateMexesFlag = 0

function getNameFromBp(bp)
	local techLevel = false
    local levels = {TECH1 = 1,TECH2 = 2,TECH3 = 3}
    for i, v in bp.Categories do
        if levels[v] then
            techLevel = levels[v]
            break
        end
    end
    if techLevel then
        return LOCF("T%d %s", techLevel, bp.Description)
	else
		return LOC(bp.Description)
    end
end

function getMassProducedWithoutMS(unit)
	if EntityCategoryContains(categories.TECH1, unit) then
		return 2
	elseif EntityCategoryContains(categories.TECH2, unit) then
		return 6
	elseif EntityCategoryContains(categories.TECH3, unit) then
		return 18
	else
		print("Unknown tech level for mass extractor!")
	end
end

function insertMex(mextable, unit)
	if unit:GetFocus() and EntityCategoryContains(categories.MASSEXTRACTION, unit:GetFocus()) then
		table.insert(mextable.U.Y, unit)
	else
		table.insert(mextable.U.N, unit)
	end
	
	local econoData = unit:GetEconData()
	--print (econoData.massProduced .. "." .. getMassProducedWithoutMS(unit) .. "." .. repr(econoData.massProduced <= getMassProducedWithoutMS(unit)))
	if econoData.energyRequested == econoData.energyConsumed and 
	   econoData.massProduced <= getMassProducedWithoutMS(unit) and
	   econoData.energyRequested > 0 then
		table.insert(mextable.MS.N, unit)
	end
end

-- this is not super precise, if you have a damaged disabled mex it will be counted as beeing built
function isMexBeingBuilt(mex)
	if mex:GetEconData().energyRequested != 0 then
		return false
	end
		
	if mex:GetHealth() == mex:GetMaxHealth() then
		return false
	end
	
	return true
end

function mexesThatAreNotBeeingBuilt(mexes)
	local newMexes = {}
	for _, mex in mexes do
		if not isMexBeingBuilt(mex) then
			table.insert(newMexes, mex)
		end
	end
	return newMexes
end

function updateButton(mexTable, button, enableMarker, tooltipText, countBeingBuilt)
	if not countBeingBuilt then
		mexTable = mexesThatAreNotBeeingBuilt(mexTable)
	end
	
	local aliveCount = table.getsize(mexTable)
	
	if aliveCount > 0 then
		local unitBluePrint = mexTable[1]:GetBlueprint()
		ToolTip.AddControlTooltip(button, {text=getNameFromBp(unitBluePrint), body=tooltipText})
		
		-- assign units that will be selected when the button is clicked
		button.units = mexTable
		button.count:SetText(aliveCount)
		
		-- set the texture that corresponds to the unit
		local iconName1, iconName2, iconName3, iconName4 = GameCommon.GetCachedUnitIconFileNames(unitBluePrint)
		button.icon:SetTexture(iconName1)
		
		-- show the button
		button:Show()
		
		-- show work progress on mexes that are being updated
		if enableMarker then
			local maxWorkProgress = 0
			for _, mex in mexTable do
				if mex:GetWorkProgress() > maxWorkProgress then
					maxWorkProgress = mex:GetWorkProgress()
				end
			end
			button.progress.Height:Set(3)
			button.progress:SetValue(maxWorkProgress)
		else
			button.progress.Height:Set(0)
		end
		
		-- show the marker
		if enableMarker then
			button.marker:Show()
		else
			button.marker:Hide()
		end
		
	else
		button:Hide()
	end
end

--remove dead fabs from the list
function cleanunitList()

	deadCount = 0

	for i, unit in units do
		if unit:IsDead() then
			units[i] = nil
			deadCount = 1
		end
	end
	
	if deadCount > 0 then
		DelayUpdateMexes = 1
	end

end

function UpdateMexesList()
		
		units = {}
		
		local selection = GetSelectedUnits()
		
	
		local mflist = UISelectionByCategory("MASSEXTRACTOR * STRUCTURE", false, false, false, false)
		local mflist = GetSelectedUnits() or {}
		
		for i, unit in mflist do
			units[unit:GetEntityId()] = unit
		end	
		
		SelectUnits(selection)
		
		UpdateOrderedMexesList()
		
end

--auto update the fab list
-- AutoUpdateMexesList = function()
-- 	while true do
-- 		local selection = GetSelectedUnits()
-- 		if selection then
-- 			WaitSeconds(0.3)
-- 		else
-- 			units = {}
-- 			
-- 			local mflist = UISelectionByCategory("MASSEXTRACTOR * STRUCTURE", false, false, false, false)
-- 			local mflist = GetSelectedUnits() or {}
-- 			
-- 			for i, unit in mflist do
-- 				units[unit:GetEntityId()] = unit
-- 			end	
-- 			
-- 			SelectUnits(selection)
-- 			
-- 			UpdateOrderedMexesList()

-- 			WaitSeconds(30)
-- 		end
-- 	end
-- end

function UpdateOrderedMexesList()
		cleanunitList()

		mexes[categories.TECH1] = {MS = {Y = {}, N = {}}, U = {Y = {}, N = {}}}
		mexes[categories.TECH2] = {MS = {Y = {}, N = {}}, U = {Y = {}, N = {}}}
		mexes[categories.TECH3] = {MS = {Y = {}, N = {}}, U = {Y = {}, N = {}}}
		
		for index, unit in units do
			if EntityCategoryContains(categories.MASSEXTRACTION, unit) then
				if EntityCategoryContains(categories.TECH1, unit) then
					insertMex(mexes[categories.TECH1], unit)
				elseif EntityCategoryContains(categories.TECH2, unit) then
					insertMex(mexes[categories.TECH2], unit)
				elseif EntityCategoryContains(categories.TECH3, unit) then
					insertMex(mexes[categories.TECH3], unit)
				else
					print("Unknown tech level for mass extractor!")
				end
			end
		end
end

function CreateButton(parent)
   -- create the button
   local myButton = Button(parent, UIUtil.UIFile('/mods/'.. modFolder .. '/textures/add_down.dds'), UIUtil.UIFile('/mods/'.. modFolder .. '/textures/add_up.dds'), UIUtil.UIFile('/mods/'.. modFolder .. '/textures/add_down.dds'), UIUtil.UIFile('/mods/'.. modFolder .. '/textures/add_over.dds'))
   LayoutHelpers.AtLeftTopIn(myButton, parent, 20, 560) 
   myButton.Depth:Set(4)

   -- onclick handler
   myButton.OnClick = function()
   	  UpdateMexesList()
   end
   
     -- create the button
    local myButton2 = Checkbox(parent, '/mods/'.. modFolder .. '/textures/addall_up.dds', '/mods/'.. modFolder .. '/textures/addall_down.dds', '/mods/'.. modFolder .. '/textures/addall_over.dds', '/mods/'.. modFolder .. '/textures/addall_over.dds', '/mods/'.. modFolder .. '/textures/addall_up.dds', '/mods/'.. modFolder .. '/textures/addall_up.dds', "UI_Menu_MouseDown_Sml", "UI_Menu_MouseDown_Sml")
    LayoutHelpers.AtLeftTopIn(myButton2, parent, 40, 560) 
    myButton2.Depth:Set(4)

   -- onclick handler
	myButton2.OnClick = function()
		if myButton2:IsChecked() then
			AutoUpdateMexesFlag = 0
			myButton2:SetCheck(false)
			print("Autoupdate massextractors disabled.")
		else
			AutoUpdateMexesFlag = 1
			myButton2:SetCheck(true)
			print("Autoupdate massextractors enabled (every 30 seconds).")
		end
   end
   
   

end

function CreateModUI(isReplay, parent)
	local xPosition = 20
	local yPosition = 420
	
	local buttons = CreateGrid(parent, xPosition, yPosition, 3, 3, CreateGenericButton)

	for tech = 1, 3 do
		img = Bitmap(parent)
		img:SetTexture(UIUtil.UIFile('/game/avatar-engineers-panel/tech-'.. tech .. '_bmp.dds'))
		LayoutHelpers.CenteredLeftOf(img, buttons[1][tech], -6)
	end
	
	function UpdateMexes()
		UpdateOrderedMexesList()
		
		updateButton(mexes[categories.TECH1].U.N, buttons[1][1], false, "Select idle T1 Mexes", false)
		updateButton(mexes[categories.TECH1].U.Y, buttons[2][1], true, "Select T1 Mexes that are being updated", false)
		buttons[3][1]:Hide()
		updateButton(mexes[categories.TECH2].U.N, buttons[1][2], false, "Select idle T2 Mexes", false)
		updateButton(mexes[categories.TECH2].U.Y, buttons[2][2], true, "Select T2 Mexes that are being updated", false)
		updateButton(mexes[categories.TECH2].MS.N, buttons[3][2], false, "Select T2 Mexes that are not surrounded by mass storage", true)
		buttons[1][3]:Hide()
		buttons[2][3]:Hide()
		updateButton(mexes[categories.TECH3].U.N, buttons[1][3], false, "Select idle T3 Mexes", false)
		updateButton(mexes[categories.TECH3].MS.N, buttons[3][3], false, "Select T3 Mexes that are not surrounded by mass storage", true)
	end
 
	CreateButton(parent)
	
	--GameMain.AddBeatFunction(UpdateMexes)
	ForkThread(UpdateMexesThread)
end

UpdateMexesThread = function()
	while true do
		for counter = 1, 4 do
			if AutoUpdateMexesFlag > 0 then
				AutoUpdateMexesFlag = AutoUpdateMexesFlag + 1
				if AutoUpdateMexesFlag > 60 then
					DelayUpdateMexes = 1
				end
			end
		
			if DelayUpdateMexes > 0 then
			
		 		local selection = GetSelectedUnits()
		 		if not selection then
					if AutoUpdateMexesFlag > 60 then
						AutoUpdateMexesFlag = 1
					end
					DelayUpdateMexes = 0
					
					UpdateMexesList()
				end
				
			end
			
			WaitSeconds(0.5)
		end	
		UpdateMexes()
	end
end
